#include "SmartDial.hpp"

SmartDial::SmartDial(int pin)
{
    m_pin = pin;
}
// Returns a value between 0 and 6 (inclusive)
int SmartDial::getValue()
{
    int v = analogRead(m_pin);
    int mapped = map(v, 30, 1024, 0, 6);
    return mapped;
}

// Returns whether the value is currently increasing (1 to 3), decreasing (-1 to -3) or stable (0).
int SmartDial::smoothEdit(int *out_value, int t_min, int t_max)
{
    bool edit = false;
    if (millis() >= next_step_time)
    {
        next_step_time = millis() + m_interval;
        edit = true;
    }

    int value = getValue();
    m_interval = 500;
    if (value < 3)
    {
        if (edit && *out_value - 1 >= t_min)
            (*out_value)--;
        int speedSetting = (value + 3) - 6; // -3, -2, -1
        m_interval = getDelayFromSpeed(speedSetting);
        return speedSetting;
    }
    if (value > 3)
    {
        if (edit && *out_value + 1 <= t_max)
            (*out_value)++;
        int speedSetting = (value - 3); // 3, 2, 1
        m_interval = getDelayFromSpeed(speedSetting);
        return speedSetting;
    }
    return 0;
}

int SmartDial::getDelayFromSpeed(int speedSetting)
{
    switch (abs(speedSetting))
    {
    case 1:
        return 1000;
    case 2:
        return 500;
    case 3:
        return 100;
    }
    return 0;
}